@@ -627,7 +627,7 @@ int gettok(char *token, int toksiz)
 {
 	int t, i;
 	int tok;
-	char name[MAXNAME];
+	char name[MAXNAME+1];
 
 	for ( ; level >= 0; level--) {
 		for (tok = deftok(token, toksiz, infile[level]); tok != EOF;
@@ -643,7 +643,7 @@ int gettok(char *token, int toksiz)
 			}
 			else if (equal(token, incl) == NO)
 				return(tok);
-			for (i = 0 ;; i = strlen((char *) (&name[0]))) {
+			for (i = 0; i < MAXNAME - 1; i = strlen(name)) {
 				t = deftok(&name[i], MAXNAME, infile[level]);
 				if (t == NEWLINE || t == SEMICOL) {
 					pbstr(&name[i]);
@@ -846,8 +846,10 @@ int gtok(char *lexstr, int toksiz, FILE *fd)
 		 || c == BANG || c == CARET || c == EQUALS
 		 || c == AND || c == OR)
 		i = relate(lexstr, fd);
-	if (i >= toksiz-1)
+	if (i >= toksiz-1) {
 		synerr("token too long.");
+		i = toksiz-2;
+	}
 	lexstr[i+1] = EOS;
 	if (lexstr[0] == NEWLINE)
 		linect[level] = linect[level] + 1;
@@ -909,19 +911,27 @@ int lex(char *lexstr)
 	return(tok);
 }
 
+char getc_or_pop(FILE *fd)
+{
+	char c;
+
+	if (bp >= 0) {
+		c = buf[bp];
+		bp--;
+	} else {
+		c = getc(fd);
+	}
+
+	return c;
+}
+
 /*
  * ngetch - get a (possibly pushed back) character
  *
  */
 char ngetch(char *c, FILE *fd)
 {
-
-	if (bp >= 0) {
-		*c = buf[bp];
-		bp--;
-	}
-	else
-		*c = (char) getc(fd);
+        *c = getc_or_pop(fd);
 
 /*
  *					check for a continuation '_\n'
@@ -930,12 +940,7 @@ char ngetch(char *c, FILE *fd)
  */
 	while ( *c == UNDERLINE)
 	{
-		if (bp >= 0) {
-			*c = buf[bp];
-			bp--;
-		}
-		else
-			*c = (char) getc(fd);
+		*c = getc_or_pop(fd);
 
 		if (*c != NEWLINE)
 		{
@@ -947,12 +952,7 @@ char ngetch(char *c, FILE *fd)
 		{
 			while(*c == NEWLINE)
 			{
-				if (bp >= 0) {
-					*c = buf[bp];
-					bp--;
-				}
-				else
-					*c = (char) getc(fd);
+				*c = getc_or_pop(fd);
 			}
 		}
 	}
